"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const automation_name_1 = require("./automation-name");
const capabilities_helper_1 = require("./capabilities-helper");
const utils_1 = require("./utils");
const fs_1 = require("fs");
class NsCapabilities {
    constructor(_parser) {
        this._parser = _parser;
        this._exceptions = new Array();
        this._projectDir = this._parser.projectDir;
        this._projectBinary = this._parser.projectBinary;
        this._pluginRoot = this._parser.pluginRoot;
        this._pluginBinary = this._parser.pluginBinary;
        this._appPath = this._parser.appPath;
        this._port = this._parser.port;
        this._verbose = this._parser.verbose;
        this._appiumCapsLocation = this._parser.appiumCapsLocation;
        this._relaxedSecurity = this._parser.relaxedSecurity;
        this._cleanApp = this._parser.cleanApp;
        this._attachToDebug = this._parser.attachToDebug;
        this._sessionId = this._parser.sessionId;
        this._startSession = this._parser.startSession;
        this._testFolder = this._parser.testFolder;
        this._storage = this._parser.storage;
        this._testReports = this._parser.testReports;
        this._devMode = this._parser.devMode;
        this._runType = this._parser.runType;
        this._isSauceLab = this._parser.isSauceLab;
        this._ignoreDeviceController = this._parser.ignoreDeviceController;
        this._wdaLocalPort = this._parser.wdaLocalPort;
        this._path = this._parser.path;
        this._capabilitiesName = this._parser.capabilitiesName;
        this._imagesPath = this._parser.imagesPath;
    }
    get path() { return this._path; }
    get projectDir() { return this._projectDir; }
    get projectBinary() { return this._projectBinary; }
    get pluginRoot() { return this._pluginRoot; }
    get pluginBinary() { return this._pluginBinary; }
    get port() { return this._port; }
    set port(port) { this._port = port; }
    get verbose() { return this._verbose; }
    set verbose(verbose) { this._verbose = verbose; }
    get appiumCapsLocation() { return this._appiumCapsLocation; }
    get appiumCaps() { return this._appiumCaps; }
    set appiumCaps(appiumCaps) { this._appiumCaps = appiumCaps; }
    get testFolder() { return this._testFolder; }
    get storage() { return this._storage; }
    get testReports() { return this._testReports; }
    get reuseDevice() { return this._reuseDevice; }
    get devMode() { return this._devMode; }
    get runType() { return this._runType; }
    get isAndroid() { return this.isAndroidPlatform(); }
    get isIOS() { return !this.isAndroid; }
    get isSauceLab() { return this._isSauceLab; }
    get automationName() { return this._automationName; }
    set automationName(automationName) {
        this._automationName = automationName;
    }
    get appPath() { return this._appPath; }
    get appName() { return this._appName; }
    set appName(appName) { this._appName = appName; }
    get ignoreDeviceController() { return this._ignoreDeviceController; }
    set ignoreDeviceController(ignoreDeviceController) { this._ignoreDeviceController = ignoreDeviceController; }
    get wdaLocalPort() { return this._wdaLocalPort; }
    get device() { return this._device; }
    set device(device) { this._device = device; }
    get emulatorOptions() { return (this._emulatorOptions || "-wipe-data -gpu on"); }
    get relaxedSecurity() { return this._relaxedSecurity; }
    get cleanApp() { return this._cleanApp; }
    get attachToDebug() { return this._attachToDebug; }
    get sessionId() { return this._sessionId; }
    set sessionId(sessionId) { this._sessionId = sessionId; }
    get startSession() { return this._startSession; }
    get deviceManager() { return this._deviceManager; }
    set deviceManager(deviceManager) { this._deviceManager = deviceManager; }
    get isValidated() { return this._isValidated; }
    get imagesPath() { return this._imagesPath; }
    //set isValidated(isValidated: boolean) { this._isValidated = isValidated; }
    setAutomationNameFromString(automationName) {
        const key = Object.keys(automation_name_1.AutomationName).filter((v, i, a) => v.toLowerCase() === automationName.toLowerCase());
        this._automationName = automation_name_1.AutomationName[key[0]];
    }
    extend(args) {
        Object.keys(args).forEach(key => {
            if (args[key]) {
                this[`_${key}`] = args[key];
            }
        });
        return this;
    }
    validateArgs() {
        if (this._attachToDebug || this._sessionId) {
            this._isValidated = true;
        }
        if (!this._attachToDebug && !this._sessionId) {
            this._appiumCaps = capabilities_helper_1.resolveCapabilities(this.appiumCapsLocation, this.runType, this.projectDir, this._capabilitiesName);
            this.setAutomationName();
            this.resolveApplication();
            this.checkMandatoryCapabiliies();
            this.throwExceptions();
            this.shouldSetFullResetOption();
            this._isValidated = true;
        }
        else {
            this._isValidated = false;
        }
    }
    isAndroidPlatform() { return this._appiumCaps && this._appiumCaps ? this._appiumCaps.platformName.toLowerCase().includes("android") : undefined; }
    shouldSetFullResetOption() {
        if (!this._ignoreDeviceController) {
            this._reuseDevice = !this.appiumCaps["fullReset"];
            this.appiumCaps["fullReset"] = false;
            if (!this.reuseDevice) {
                utils_1.logWarn("The started device will be killed after the session!");
                utils_1.logInfo("To avoid it, set 'fullReset: false' in appium capabilities.");
            }
            this._cleanApp = !this.appiumCaps["noReset"];
        }
        if (this._attachToDebug || this._devMode) {
            this.appiumCaps["fullReset"] = false;
            this.appiumCaps["noReset"] = true;
            utils_1.logInfo("Changing appium setting fullReset: false and noReset: true ");
        }
    }
    setAutomationName() {
        if (this.appiumCaps["automationName"]) {
            switch (this.appiumCaps["automationName"].toLowerCase()) {
                case automation_name_1.AutomationName.UiAutomator2.toString().toLowerCase():
                    this._automationName = automation_name_1.AutomationName.UiAutomator2;
                    break;
                case automation_name_1.AutomationName.Appium.toString().toLowerCase():
                    this._automationName = automation_name_1.AutomationName.Appium;
                    break;
                case automation_name_1.AutomationName.XCUITest.toString().toLowerCase():
                    this._automationName = automation_name_1.AutomationName.XCUITest;
                    break;
            }
        }
        else {
            if (this.isAndroid) {
                if (this.tryGetAndroidApiLevel() > 6 || (this.appiumCaps["apiLevel"] && this.appiumCaps["apiLevel"].toLowerCase().includes("p"))) {
                    this._automationName = automation_name_1.AutomationName.UiAutomator2;
                }
            }
        }
        if (this._automationName) {
            this.appiumCaps["automationName"] = this._automationName.toString();
            utils_1.logInfo(`Automation name set to: ${this.appiumCaps["automationName"]}`);
            console.log(`To change automation name, you need to set it in appium capabilities!`);
        }
        else {
            console.log(`Appium will use default automation name`);
        }
    }
    tryGetAndroidApiLevel() {
        try {
            if (this.appiumCaps["platformVersion"]) {
                const apiLevel = this.appiumCaps["platformVersion"].split(".").splice(0, 2).join('.');
                return parseFloat(apiLevel);
            }
        }
        catch (error) { }
        return undefined;
    }
    resolveApplication() {
        if (this._isSauceLab) {
            if (this._appPath) {
                this._appiumCaps.app = this._appPath.startsWith("http") ? this._appPath : `sauce-storage:${this._appPath}`;
            }
            if (!this._appiumCaps.app) {
                const errorMsg = "Neither appPath option nor capabilities.app provided!!!";
                utils_1.logError(errorMsg);
                throw new Error(errorMsg);
            }
            this._appPath = this._appiumCaps.app;
            this._ignoreDeviceController = true;
            utils_1.logInfo(`Using Sauce Labs. The application path is changed to: ${this._appiumCaps.app}`);
        }
        else {
            this._appiumCaps.app = utils_1.getAppPath(this);
            this._appPath = this._appiumCaps.app;
            utils_1.logInfo(`Application full path: ${this._appiumCaps.app}`);
        }
    }
    checkMandatoryCapabiliies() {
        const appPackage = this.isAndroid ? "appPackage" : "bundleId";
        if (!this.isSauceLab && (!this._appiumCaps[appPackage] && !fs_1.existsSync(this._appiumCaps.app))) {
            this._exceptions.push(`The application folder doesn't exists or no ${appPackage} provided!`);
        }
        if (!this._runType) {
            this._exceptions.push("Missing runType! Please select one from appium capabilities file!");
        }
        if (!this._appiumCaps.platformName) {
            this._exceptions.push("Platform name is missing! Please, check appium capabilities file!");
        }
        if (!this._appiumCaps.platformVersion) {
            utils_1.logWarn("Platform version is missing! You'd better to set it in order to use the correct device");
        }
        if (!this._appiumCaps.deviceName && !this._appiumCaps.udid) {
            this._exceptions.push("The device name or udid are missing! Please, check appium capabilities file!");
        }
    }
    throwExceptions() {
        this._exceptions.forEach(msg => {
            utils_1.logError(msg);
        });
        if (this._exceptions.length > 0) {
            process.exit(1);
        }
    }
}
exports.NsCapabilities = NsCapabilities;
//# sourceMappingURL=ns-capabilities.js.map